/*
 * strdict.h
 *
 * string dictionary
 * hashed storage, with skiplist buckets
 * header file
 */

#ifndef _STRDICT_INC

#define _STRDICT_INC

#define DEFAULT_BUCKETS 23 /* default buckets per dict- keep it prime! */
#define MAX_BUCKETS 57 /* never create a dictionary with more than this # of buckets */
#define MAX_LEVEL 10 /* max level for skiplist nodes */
 
struct t_SkipNode {
	char *key;
	caddr_t data;
	struct t_SkipNode **forward;
};
typedef struct t_SkipNode SkipNode;

struct t_SkipList {
	int level;
	SkipNode *header;
	int (*compare)();
	void (*assign)();
};
typedef struct t_SkipList SkipList;

struct t_Dictionary {
	int buckets;
	SkipList *bucket[MAX_BUCKETS];
};
typedef struct t_Dictionary *string_dict;
typedef struct t_Dictionary *int_dict;

typedef SkipList *simple_dict;

/* exported functions */
extern string_dict new_dictionary(void);
extern int_dict new_int_dictionary(void);
extern void destroy_dict(string_dict d);
extern void destroy_int_dict(int_dict d);
extern void set_dict(string_dict d, char *key, caddr_t data);
extern void set_int_dict(int_dict d, int key, caddr_t data);
extern void delete_from_dict(string_dict d, char *key);
extern void delete_from_int_dict(int_dict d, int key);
extern caddr_t lookup_dict(string_dict d, char *key);
extern caddr_t lookup_int_dict(int_dict d, int key);
extern void traverse_dict(string_dict d, void (*proc)(), caddr_t rock);
extern void assign_string(char **dest, char *src);
extern string_dict new_dictionary_nbuckets(int buckets);
extern simple_dict new_simple_dict(void);
extern void simple_dict_traverse(simple_dict l, void (*proc)(), caddr_t rock);
extern void simple_dict_delete(simple_dict list, char *key);
extern void simple_dict_set(simple_dict list, char *key, caddr_t data);
extern void simple_dict_destroy(simple_dict list);
extern caddr_t simple_dict_lookup(simple_dict l, char *key);

#endif /* _STRDICT_INC */

